{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
OSCPM-ASPEL2 module for the osCommerce Product Manager for Windows
(oscpm1_aspel). Copyright 2004-2006 by Mario A. Valdez-Ramirez.

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, DBTables, IdBaseComponent, IdComponent, IdTCPServer,
  IdAntiFreezeBase, IdAntiFreeze, SyncObjs, ComCtrls;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Using the DB file (o1as_AspelDBFile), open the table (o1as_AspelDBTable)
and search for the received values in the input field (o1as_AspelInputField)
and returns the output field (o1as_AspelOutputField).

For the TCP connection with the client, use the given TCP port (o1as_AspelTCPPort).

If asked (o1as_AspelShowAll), show a detailed status panel about the transactions ().

If asked, convert the output value using the following parameters:
For each found record (when searching the input value), get the conversion field
(o1as_AspelConvField) and search its value in the conversion table
(o1as_AspelConvDBTable) in the conversion input field (o1as_AspelConvInputField)
and return its conversion output value (o1as_AspelConvOutputField).
The output field will be multiplied by the conversion output value to get the
normalized return value.
Then search the conversion table again looking for the wanted conversion units
(o1as_AspelConvWantedLookup) in the conversion input field, get the conversion
output value and divide the normalized return value by it.
The return the value to the client.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}




CONST
  o1as_AppName = 'Sincronizador de OSCPM1-ASPEL II';
  o1as_AppShortName = 'oscpm1-aspel-sync2';
  o1as_Version = '0.2.1';
  o1as_Homepage = 'http://www.mariovaldez.net/software/oscpmwin/modules/';
  o1as_Copyright = 'Copyright 2004-2006 by Mario A. Valdez-Ramirez';
  o1as_Email = 'mario@mariovaldez.org';
  o1as_TCPPort = 27572;
  o1as_TCPHost = '127.0.0.1';
  o1as_TCPTimeout = 30000;
  o1as_TCPMaxChars = 250;
  o1as_MaxSimConnections = 10;
  o1as_QuitCommand = 'QUIT OSCPM1';
  o1as_StatCommand = 'STATS OSCPM1';
  o1as_OKCode = 'OK';
  o1as_StatCode = 'STAT';
  o1as_StatCodeEnd = 'STAT END';
  o1as_ModNACode = 'NOT FOUND';
  o1as_ModErrorCode = 'ERROR ERROR';
  o1as_GaugeBucket = 25;
  o1as_StatTotalReqs = 'Consultas totales';
  o1as_StatMatchingReqs = 'Consultas coincidentes';
  o1as_StatNoMatchReqs = 'Consultas sin coincidentes';
  o1as_StatErrorReqs = 'Consultas con error';
  o1asC_Found = 1;
  o1asC_NotFound = 2;
  o1asC_Error = 3;
  o1asC_Total = 4;
  o1asC_ShowPanel = 'P';
  o1asC_ShowDebug = 'D';
  o1asC_ShowExch = 'X';


TYPE
  To1as_Form_Main = class(TForm)
    o1as_GroupBox_Main: TGroupBox;
    o1as_label_conninfo: TLabel;
    o1as_label_recinfo: TLabel;
    o1as_label_dbinfo: TLabel;
    o1as_TCPServer_Main: TIdTCPServer;
    o1as_label_connstat: TLabel;
    o1as_GroupBox_Stats: TGroupBox;
    o1as_label_totalreqs: TLabel;
    o1as_label_totalreqsLeg: TLabel;
    o1as_label_recfoundLeg: TLabel;
    o1as_label_recfound: TLabel;
    o1as_label_recnotfound: TLabel;
    o1as_label_recnotfoundLeg: TLabel;
    o1as_label_recerrorLeg: TLabel;
    o1as_label_recerror: TLabel;
    o1as_label_dbinfo2: TLabel;
    o1as_label_convfactor: TLabel;
    o1as_IdAntiFreeze_Main: TIdAntiFreeze;
    o1as_label_debug: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure o1as_TCPServer_MainExecute(AThread: TIdPeerThread);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  public
    PROCEDURE o1as_PRChangeTotalConns (NumDelta : LONGINT);
    PROCEDURE o1as_PRShowDebugLine (DebugStr : STRING);
    FUNCTION o1as_FNMakeConsult (SearchedData : STRING; VAR DBQuery : TQuery) : STRING;
    FUNCTION o1as_FNGetProductItem (IndexData : STRING; VAR DBQuery : TQuery) : STRING;
    FUNCTION o1as_FNGetConversionLookup (WishedData : STRING; VAR DBQuery : TQuery) : STRING;
    PROCEDURE o1as_PRShowConnStatRec;
    PROCEDURE o1as_PRShowConnStatFin;
    PROCEDURE o1as_PRCloseAll (Sender: TObject);
    PROCEDURE o1as_PRChangeCounter (CounterType, NumDelta : LONGINT);
  end;

  o1as_QueryCounterRecord = RECORD
                              ThreadID : CARDINAL;
                              TotalReqs,
                              RecNotFound,
                              RecFound,
                              RecError : LONGINT;
                            END;
  o1as_QueryCounterList = ARRAY [0..o1as_MaxSimConnections] OF o1as_QueryCounterRecord;


VAR
  o1as_Form_Main: To1as_Form_Main;
  o1as_DBLock : TCriticalSection;
  o1as_UILock : TCriticalSection;
  o1as_GVLock : TCriticalSection;
  o1asG_RunningLinks : LONGINT;
  o1asG_AspelDBFile : STRING;
  o1asG_AspelDBTable : STRING;
  o1asG_AspelInputField : STRING;
  o1asG_AspelOutputField : STRING;
  o1asG_AspelTCPPort : LONGINT;
  o1asG_AspelShowPanel : BOOLEAN;
  o1asG_AspelShowDebug : BOOLEAN;
  o1asG_AspelShowData : BOOLEAN;
  o1asG_QueryCounters : o1as_QueryCounterRecord;
  o1asG_AspelConvField : STRING;
  o1asG_AspelConvDBTable : STRING;
  o1asG_AspelConvInputField : STRING;
  o1asG_AspelConvOutputField : STRING;
  o1asG_AspelConvWantedLookup : STRING;
  o1asG_AspelDoConversion : BOOLEAN;
  o1asG_AspelConvWantedVal : STRING;



implementation

{$R *.dfm}

uses IdSocketHandle;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.FormCreate (Sender: TObject);
VAR
  ServerBind : TIdSocketHandle;
  Query_Test : TQuery;
BEGIN
  o1asG_RunningLinks := 0;
  o1asG_AspelDBFile := TRIM (ParamStr(1));
  o1asG_AspelDBTable := TRIM (ParamStr (2));
  o1asG_AspelInputField := TRIM (ParamStr(3));
  o1asG_AspelOutputField := TRIM (ParamStr(4));
  o1asG_AspelShowPanel := (POS (o1asC_ShowPanel, UPPERCASE (TRIM (ParamStr (6)))) > 0);
  o1asG_AspelShowDebug := (POS (o1asC_ShowDebug, UPPERCASE (TRIM (ParamStr (6)))) > 0);
  o1asG_AspelShowData := (POS (o1asC_ShowExch, UPPERCASE (TRIM (ParamStr (6)))) > 0);
  o1asG_AspelConvField := TRIM (ParamStr(7));
  o1asG_AspelConvDBTable := TRIM (ParamStr(8));
  o1asG_AspelConvInputField := TRIM (ParamStr(9));
  o1asG_AspelConvOutputField := TRIM (ParamStr(10));
  o1asG_AspelConvWantedLookup := TRIM (ParamStr(11));
  o1asG_AspelDoConversion := ((o1asG_AspelConvDBTable <> '') AND (o1asG_AspelConvInputField <> '') AND (o1asG_AspelConvOutputField <> '') AND (o1asG_AspelConvField <> '') AND (o1asG_AspelConvWantedLookup <> ''));
  IF ((o1asG_AspelDBFile <> '') AND
      (o1asG_AspelDBTable <> '') AND
      (o1asG_AspelInputField <> '') AND
      (o1asG_AspelOutputField <> '')) THEN
    TRY
      Query_Test := TQuery.Create (Application);
      Query_Test.DatabaseName := o1asG_AspelDBFile;
      Query_Test.SQL.Text := 'select * from ' + o1asG_AspelDBTable;
      Query_Test.RequestLive := FALSE;
      Query_Test.UniDirectional := TRUE;
      Query_Test.Active := TRUE;
      IF (o1asG_AspelDoConversion) THEN
        BEGIN
          o1asG_AspelConvWantedVal := o1as_FNGetConversionLookup (o1asG_AspelConvWantedLookup, Query_Test);
          IF (o1asG_AspelConvWantedVal = o1as_ModErrorCode) THEN
            BEGIN
              MessageBeep (MB_ICONEXCLAMATION);
              MessageDlg ('Hubo un error al tratar de obtener el factor de conversin ' + o1asG_AspelConvWantedLookup + '. Terminando.', mtError, [mbAbort], 0);
              Application.Terminate;
            END;
        END;
    EXCEPT
      MessageBeep (MB_ICONEXCLAMATION);
      MessageDlg ('Hubo un error al abrir la base de datos de ASPEL ' + o1asG_AspelDBFile + '. Terminando.', mtError, [mbAbort], 0);
      FREEANDNIL (Query_Test);
      Application.Terminate;
    END
  ELSE
    BEGIN
      MessageBeep (MB_ICONEXCLAMATION);
      MessageDlg ('Parmetros insuficientes o invlidos. Terminando.'#13#10 +
                  'archivoMDB tabla campoindice campobuscado [puertoTCP [panel [campoconvindice tablaconv campoconvbuscado campoconvvalor convdeseada]]] ', mtError, [mbAbort], 0);
      Application.Terminate;
    END;
  SetPriorityClass (GetCurrentProcess, NORMAL_PRIORITY_CLASS);
  SetThreadPriority (GetCurrentThread, THREAD_PRIORITY_LOWEST);
  IF ((TRIM (ParamStr (5)) <> '') AND (TRIM (ParamStr (5)) <> '0')) THEN
    o1asG_AspelTCPPort := STRTOINTDEF (TRIM (ParamStr (5)), o1as_TCPPort)
  ELSE
    o1asG_AspelTCPPort := o1as_TCPPort;
  o1as_TCPServer_Main.Bindings.Clear;
  ServerBind := o1as_TCPServer_Main.Bindings.Add;
  ServerBind.Port := o1asG_AspelTCPPort;
  ServerBind.IP := o1as_TCPHost;
  o1as_TCPServer_Main.MaxConnections := o1as_MaxSimConnections;
  TRY
    o1as_TCPServer_Main.Active := TRUE;
  EXCEPT
  END;
  IF (o1as_TCPServer_Main.Active = FALSE) THEN
    BEGIN
      MessageBeep (MB_ICONEXCLAMATION);
      MessageDlg ('No se puede establecer el enlace ' + o1as_TCPHost + ':' + INTTOSTR (o1asG_AspelTCPPort) + ' con la aplicacin principal. Terminando.', mtError, [mbAbort], 0);
      Application.Terminate;
    END;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.FormShow(Sender: TObject);
begin
  o1as_Form_Main.Caption := o1as_AppName + ' ' + o1as_Version;
  o1as_GroupBox_Main.Caption := '';
  o1as_GroupBox_Stats.Caption := '';
  o1as_label_dbinfo.Caption := o1asG_AspelDBFile;
  o1as_label_dbinfo2.Caption := o1asG_AspelInputField + ' -> ' + o1asG_AspelOutputField + '@' + o1asG_AspelDBTable;
  o1as_label_conninfo.Caption := 'Esperando conexiones en ' + o1as_TCPHost + ':' + INTTOSTR (o1asG_AspelTCPPort);
  o1as_label_recinfo.Caption := '';
  o1as_label_connstat.Caption := 'Esperando conexiones.';
  o1as_label_debug.Caption := '';
  IF (o1asG_AspelDoConversion = TRUE) THEN
    o1as_label_convfactor.Caption := o1asG_AspelConvField + ' -> ' + o1asG_AspelConvInputField + ' -> ' + o1asG_AspelConvOutputField + '@' + o1asG_AspelConvDBTable + ' [*' + o1asG_AspelConvWantedVal + ']'
  ELSE
    o1as_label_convfactor.Caption := 'Sin conversin';
  o1as_label_totalreqs.Caption := '0';
  o1as_label_recfound.Caption := '0';
  o1as_label_recnotfound.Caption := '0';
  o1as_label_recerror.Caption := '0';
  o1as_GroupBox_Main.Visible := o1asG_AspelShowPanel;
  IF (o1asG_AspelShowPanel = TRUE) THEN o1as_GroupBox_Stats.Top := (o1as_GroupBox_Main.Height - 1);
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.o1as_TCPServer_MainExecute(AThread: TIdPeerThread);
VAR
  Query_Main : TQuery;
  CurrInputItem : STRING;
  CurrOutputItem : STRING;
begin
  IF (o1asG_RunningLinks < o1as_MaxSimConnections) THEN
    BEGIN
      SetThreadPriority (GetCurrentThread, THREAD_PRIORITY_LOWEST);
      o1as_PRChangeTotalConns (1);
      AThread.OnTerminate := o1as_PRCloseAll;
      CurrInputItem := '';
      o1as_PRShowConnStatRec;

      Query_Main := TQuery.Create (NIL);
      Query_Main.DatabaseName := o1asG_AspelDBFile;
      Query_Main.SQL.Text := 'select * from ' + o1asG_AspelDBTable;
      Query_Main.RequestLive := TRUE;
      Query_Main.UniDirectional := TRUE;
      o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Database connection set.' );
      TRY
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_AppName + ' ' + o1as_Version);
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_Homepage);
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_Copyright);
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_Email);
        o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Welcome headers sent.' );
        REPEAT
          SLEEP (1);
          o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Database connection set.' );
          CurrInputItem := AThread.Connection.ReadLn (#13#10, o1as_TCPTimeout, o1as_TCPMaxChars);
          CurrInputItem := TRIM (AdjustLineBreaks (CurrInputItem));
          o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Received query:' + CurrInputItem);
          IF (UPPERCASE (CurrInputItem) = o1as_QuitCommand) THEN
            BEGIN
              o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Disconnecting.');
              AThread.Connection.Disconnect;
              BREAK;
            END
          ELSE IF (UPPERCASE (CurrInputItem) = o1as_StatCommand) THEN
            BEGIN
              o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Returning statistics.');
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatTotalReqs + ': ' + INTTOSTR (o1asG_QueryCounters.TotalReqs));
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatMatchingReqs + ': ' + INTTOSTR (o1asG_QueryCounters.RecFound));
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatNoMatchReqs + ': ' + INTTOSTR (o1asG_QueryCounters.RecNotFound));
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatErrorReqs + ': ' + INTTOSTR (o1asG_QueryCounters.RecError));
              AThread.Connection.WriteLn (o1as_StatCodeEnd);
            END
          ELSE
            BEGIN
              o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Processing query.');
              CurrOutputItem := o1as_FNMakeConsult (CurrInputItem, Query_Main);
              o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Returning query.');
              AThread.Connection.WriteLn (o1as_OKCode + ' ' + CurrOutputItem);
              IF (o1asG_AspelShowData) THEN AThread.Connection.WriteLn (o1as_StatCode + ' ' + CurrInputItem + ' <-> ' + CurrOutputItem);
            END;
        UNTIL ((UPPERCASE (CurrInputItem) = o1as_QuitCommand) OR (CurrInputItem = ''));
      FINALLY
        o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' Excepting, disconnecting.');
        AThread.Connection.Disconnect;
        FREEANDNIL (Query_Main);
      END;
      o1as_PRShowConnStatFin;
    END
  ELSE
    BEGIN
      o1as_PRShowDebugLine (INTTOSTR (AThread.ThreadID) + ' MaxConnLimit reached. Disconnecting.');
      AThread.Connection.Disconnect;
    END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRChangeTotalConns (NumDelta : LONGINT);
BEGIN
  o1as_GVLock.Acquire;
  TRY
    o1asG_RunningLinks := o1asG_RunningLinks + NumDelta;
  FINALLY
    o1as_GVLock.Release;
  END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRShowDebugLine (DebugStr : STRING);
BEGIN
  IF (o1asG_AspelShowDebug) THEN
    BEGIN
      o1as_UILock.Acquire;
      TRY
        o1as_label_debug.Caption := DebugStr;
      FINALLY
        o1as_UILock.Release;
      END;
    END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION To1as_Form_Main.o1as_FNMakeConsult (SearchedData : STRING; VAR DBQuery : TQuery) : STRING;
VAR
  CurrOutputItem : STRING;
BEGIN
  o1as_PRChangeCounter (o1asC_Total, 1);
  SearchedData := TRIM (SearchedData);
  IF (SearchedData <> '') THEN
    BEGIN
      CurrOutputItem := o1as_FNGetProductItem (SearchedData, DBQuery);
      IF (o1asG_AspelDoConversion) THEN
        TRY
          CurrOutputItem := FLOATTOSTRF ((STRTOCURR (CurrOutputItem) / STRTOCURR (o1asG_AspelConvWantedVal)), ffFixed, 15, 2);
        EXCEPT
        END;
    END
  ELSE
    BEGIN
      CurrOutputItem := o1as_ModNACode;
    END;
  o1as_FNMakeConsult := CurrOutputItem;
  o1as_UILock.Acquire;
  TRY
    IF (o1asG_AspelShowPanel = TRUE) THEN
      BEGIN
        o1as_label_recinfo.Caption := SearchedData + ' <-> ' + CurrOutputItem;
        o1as_label_connstat.Caption := 'Conexiones totales ' + INTTOSTR (o1asG_RunningLinks);
      END;
    o1as_label_totalreqs.Caption := INTTOSTR (o1asG_QueryCounters.TotalReqs);
    o1as_label_recfound.Caption := INTTOSTR (o1asG_QueryCounters.RecFound);
    o1as_label_recnotfound.Caption := INTTOSTR (o1asG_QueryCounters.RecNotFound);
    o1as_label_recerror.Caption := INTTOSTR (o1asG_QueryCounters.RecError);
  FINALLY
    o1as_UILock.Release;
  END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRShowConnStatRec;
BEGIN
  IF (o1asG_AspelShowPanel = TRUE) THEN
    BEGIN
      o1as_UILock.Acquire;
      TRY
        o1as_label_connstat.Caption := 'Recibiendo conexin ' + INTTOSTR (o1asG_RunningLinks);
      FINALLY
        o1as_UILock.Release;
      END;
    END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRShowConnStatFin;
BEGIN
  IF (o1asG_AspelShowPanel = TRUE) THEN
    BEGIN
      o1as_UILock.Acquire;
      TRY
        o1as_label_connstat.Caption := 'Conexin terminada ' + INTTOSTR (o1asG_RunningLinks);
        o1as_label_recinfo.Caption := '';
      FINALLY
        o1as_UILock.Release;
      END;
    END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION To1as_Form_Main.o1as_FNGetProductItem (IndexData : STRING; VAR DBQuery : TQuery) : STRING;
VAR
  TmpResult : STRING;
BEGIN
  TmpResult := o1as_ModNACode;
  o1as_DBLock.Acquire;
  TRY
    DBQuery.Active := FALSE;
    DBQuery.SQL.Clear;
    IF (o1asG_AspelDoConversion = FALSE) THEN
      DBQuery.SQL.Add ('select ' + o1asG_AspelOutputField + ' from ' + o1asG_AspelDBTable + ' where ' + o1asG_AspelInputField + '="' + IndexData + '";')
    ELSE
      DBQuery.SQL.Add ('select (' + o1asG_AspelDBTable+'.'+o1asG_AspelOutputField + '*' + o1asG_AspelConvDBTable+'.'+o1asG_AspelConvOutputField + ') from ' + o1asG_AspelDBTable + ' left join ' + o1asG_AspelConvDBTable + ' on trim(' + o1asG_AspelDBTable+'.'+o1asG_AspelConvField + ')=trim(' + o1asG_AspelConvDBTable+'.'+o1asG_AspelConvInputField + ') where ' + o1asG_AspelDBTable+'.'+o1asG_AspelInputField + '="' + IndexData + '";');
    TRY
      DBQuery.Active := TRUE;
      DBQuery.First;
      IF (NOT DBQuery.EOF) THEN
        BEGIN
          TmpResult := TRIM (DBQuery.Fields[0].AsString);
          o1as_PRChangeCounter (o1asC_Found, 1);
        END
      ELSE
        BEGIN
          TmpResult := o1as_ModNACode;
          o1as_PRChangeCounter (o1asC_NotFound, 1);
        END;
    EXCEPT
      TmpResult := o1as_ModErrorCode;
      o1as_PRChangeCounter (o1asC_Error, 1);
    END;
  FINALLY
    o1as_DBLock.Release;
  END;
  o1as_FNGetProductItem := TmpResult;
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION To1as_Form_Main.o1as_FNGetConversionLookup (WishedData : STRING; VAR DBQuery : TQuery) : STRING;
VAR
  TmpResult : STRING;
BEGIN
  TmpResult := o1as_ModNACode;
  o1as_DBLock.Acquire;
  TRY
    DBQuery.Active := FALSE;
    DBQuery.SQL.Clear;
    DBQuery.SQL.Add ('select ' + o1asG_AspelConvInputField + ', ' + o1asG_AspelConvOutputField + ' from ' + o1asG_AspelConvDBTable + ' where ' + o1asG_AspelConvInputField + '="' + WishedData + '";');
    TRY
      DBQuery.Active := TRUE;
      DBQuery.First;
      IF (NOT DBQuery.EOF) THEN
        TmpResult := TRIM (DBQuery.Fields[1].AsString)
      ELSE
        TmpResult := o1as_ModNACode;
    EXCEPT
      TmpResult := o1as_ModErrorCode;
    END;
  FINALLY
    o1as_DBLock.Release;
  END;
  o1as_FNGetConversionLookup := TmpResult;
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRChangeCounter (CounterType, NumDelta : LONGINT);
BEGIN
  o1as_GVLock.Acquire;
  TRY
    CASE (CounterType) OF
      o1asC_Total: o1asG_QueryCounters.TotalReqs := o1asG_QueryCounters.TotalReqs + NumDelta;
      o1asC_Found: o1asG_QueryCounters.RecFound := o1asG_QueryCounters.RecFound + NumDelta;
      o1asC_NotFound: o1asG_QueryCounters.RecNotFound := o1asG_QueryCounters.RecNotFound + NumDelta;
      o1asC_Error: o1asG_QueryCounters.RecError := o1asG_QueryCounters.RecError + NumDelta;
    END;
  FINALLY
    o1as_GVLock.Release;
  END;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  IF (o1asG_RunningLinks <= 0) THEN
    BEGIN
      o1as_TCPServer_Main.Active := FALSE;
      Action := caFree
    END
  ELSE Action := caNone;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRCloseAll (Sender: TObject);
BEGIN
  o1as_PRChangeTotalConns (-1);
  IF (o1asG_RunningLinks <= 0) THEN
    BEGIN
      o1as_TCPServer_Main.Active := FALSE;
      Application.Terminate;
    END;
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
INITIALIZATION
  o1as_DBLock := TCriticalSection.Create;
  o1as_UILock := TCriticalSection.Create;
  o1as_GVLock := TCriticalSection.Create;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FINALIZATION
  FREEANDNIL (o1as_DBLock);
  FREEANDNIL (o1as_UILock);
  FREEANDNIL (o1as_GVLock);


END.
